from abc import abstractmethod
from ...kernel.agent.Action import Action

# --------------------------------------------------------
# Define Action
# --------------------------------------------------------
class DelegateAction(Action):
    """ An action is a response to the occurrence of an event """

    def execute(self, data):
        """ 
        Response.
        @param data Event data 
        """
        self.delegate(data)
    
    def activeTimeout(self, time):
        self.adm.sendEvent(self.agent.id, 'timeout', {'time': time, 'dto': None})

    def toAssign(self, data):
        ag = self.agent.getFreeQueue().get()
        self.agent.getRequestDict()[ag] = {
            'dtoList': []
        }
        self.adm.sendEvent(ag, 'task', data)
    
    @abstractmethod
    def delegate(self, data):
        """
        Catch the exception.
        @param exception Response exception
        """
        pass




class WebAgTK():
    
    socket = None 

    def response(self, data):
        data = str(data)
        data = data.replace("'", "\"") + '\n'
        self.socket.sendall(data.encode()) 
        
    def setSocket(self, socket):
        self.socket = socket

    def updateID(self, id):
        pass



from .pooltype import PoolType
from abc import abstractmethod
from .delegate import Delegate
from .exceptions import PoolException
from ...kernel.util.Queue import Queue
from ...kernel.agent.Agent import Agent
from .responseaction import ResponseAction
from .notifyfreeaction import NotifyFreeAction

class PoolController(Agent):
    
    def __init__(self, agentID, type, bufferSize, poolSize):
        self.__type = type
        self.__poolSize = poolSize
        self.__bufferSize = bufferSize
        self.__requestDict = {}
        self.__freeQueue = Queue(poolSize)
        self.__agentList = []
        super().__init__(agentID)

    def setUp(self):
        self._social = True
        self.addBehavior('Delegate')
        if self.__type == PoolType.BLOCK:
            self.bindAction('Delegate', 'delegate', Delegate())
        self.addBehavior('Notify')
        self.bindAction('Notify', 'notify', NotifyFreeAction())
        # TODO Para el bloquenate debe ser por defecto
        # Para el no bloquenate ?
        self.addBehavior('Response')
        self.bindAction('Response', 'response', ResponseAction())
        self.build()

    def bindDelegateAction(self, action):
        if self.__type == PoolType.NO_BLOCK:
            self.bindAction('Delegate', 'delegate', action)
        else:
            raise PoolException('[Warn, bindDelegateAction]: The controller is a blocking type. No need to define delegator')

    def suscribeAgent(self, agent):
        self.__agentList.append(agent.id)
        agent.setController(self.id)
        agent.setControllerType('POOL')
        self.__freeQueue.put(agent.id)
        actions = agent.getActions()
        for action in actions:
            action.setIsPool(True)
            action.setEnableResponse(self.__type == PoolType.BLOCK)

    def suscribeRemoteAgent(self, agentID):
        if not isinstance(agentID, str):
            raise PoolException('[Warn, suscribeRemoteAgent]: The object to subscribe is not an agent ID')
        self.__agentList.append(agentID)
        self.__freeQueue.put(agentID)
    
    def broadcastEvent(self, event, data):
        from pbesa.kernel.system.Adm import Adm
        for agentID in self.__agentList:
            Adm().sendEvent(agentID, event, data)
    
    @abstractmethod
    def build(self):
        pass

    def getFreeQueue(self):
        return self.__freeQueue

    def getRequestDict(self):
        return self.__requestDict

    def getBufferSize(self):
        return self.__bufferSize

    def isBlock(self):
        return self.__type == PoolType.BLOCK



from abc import abstractmethod
from ...kernel.agent.Action import Action

# --------------------------------------------------------
# Define Action
# --------------------------------------------------------
class ResponseAction(Action):
    """ An action is a response to the occurrence of an event """
    
    def execute(self, data):
        """ 
        Response.
        @param data Event data 
        """
        if 'timeout' == data:
            results = {}
            for key, res in self.agent.getCheckDict().items():
                results[key] = res
            self.endOfProcess(results, True)
        else:
            result = 'None'
            agentID = data['source']
            if data['result']:
                result =  data['result']
            self.agent.getCheckDict()[agentID] = result
            if self.check():
                results = {}
                for key, res in self.agent.getCheckDict().items():
                    results[key] = res
                self.endOfProcess(results, False)

    def check(self):
        for res in self.agent.getCheckDict().values():
            if not res:
                return False
        return True

    def sendResponse(self, response):
        self.agent.setTimeout(False)
        self.adm.sendEvent(self.agent.id, 'timeout', {'command': 'cancel'})
        self.agent.getGateway().put(response) 
    
    @abstractmethod
    def endOfProcess(self, results, timeout):
        """
        Catch the exception.
        @param exception Response exception
        """
        pass



from abc import abstractmethod
from ...kernel.agent.Action import Action
from ...social.worker.exceptions import TaskException

# --------------------------------------------------------
# Define Action
# --------------------------------------------------------
class Task(Action):
    """ An action is a response to the occurrence of an event """

    def __init__(self):
        self.__isPool = False
        self.__enableResponse = False
        super().__init__()

    def execute(self, data):
        """ 
        Response.
        @param data Event data 
        """
        self.goHead(data)

        if self.__isPool:
            self.adm.sendEvent(self.agent.getController(), 'notify', self.agent.id)

    def activeTimeout(self, time):       
        self.adm.sendEvent(self.agent.id, 'timeout', {'time': time, 'dto': None})

    def sendResponse(self, data):
        if self.__enableResponse:
            response = {
                'source': self.agent.id,
                'result': data
            }
            self.adm.sendEvent(self.agent.getController(), 'response', response) 
        else:
            raise TaskException('[Warn, sendResponse]: The type of control does not allow synchronous responses (see Linear or Pool type Block)')
    
    @abstractmethod
    def goHead(self, data):
        """
        Catch the exception.
        @param exception Response exception
        """
        pass

    def setIsPool(self, isPool):
        self.__isPool = isPool
    
    def setEnableResponse(self, enableResponse):
        self.__enableResponse = enableResponse



from ...kernel.adapter.Adapter import Adapter
from ...kernel.io.SystemFile import SystemFile

class FileAdapter(Adapter):

    def __init__(self, config):
        self.config = config
        super().__init__()

    def setUp(self):
        if self.config['type'] == 'JSON':            
            sf = SystemFile(self.config['path'])
            self.data = sf.readJsonFile()

    def response(self):
        pass
    
    def request(self):
        return self.data

    def finalize(self):
        pass




from abc import ABC, abstractmethod

class RationalAg(ABC):

	id = None
	state = None     
	brain = None
	world = None
	exePool = None
	settings = None

	def __init__(self):
		self.settings = self.setUp()
		self.id = self.settings['id']
		self.state = self.settings['state']
		self.brain = self.settings['brain']
		self.world = self.settings['world']
		size = self.settings['pool_size']
		Adm().addAgent(self)
		self.exePool = Queue(size)        
		for x in range(1, size):
			self.exePool.put(ActionExe(self.exePool))
		super().__init__()

	@abstractmethod
	def setUp(self, settings):
		pass

	def sendEvent(self, event, data):
		self.world.update(event, data)
		actions = self.brain.derive(event, data)
		if actions:
			for action in actions:
				aExe = self.getFree()
				aExe.setAction()
				aExe.setData()
				aExe.start()

	def getFree(self):
		exeA = self.pool.get()
		self.pool.task_done()
		return exeA 




from threading import Thread
import time
import random

class ActionExe(Thread):

    let = None
    pool = None
    alive = None
    action = None

    def __init__(self, pool):
        self.let = False
        self.pool = pool
        self.alive = True
        super().__init__()

    def run(self):
        while self.alive:
            if self.let:
                self.action.execute(self.data)
                self.pool.addExe(self)                
            #time.sleep(random.random())

    def setLet(self, val):
        self.let = val

    def setAlive(self, val):
        self.alive = val

    def setAction(self, action):
    	self.action = action

    def setData(self, data):
    	self.data = data


# ----------------------------------------------------------
# Define resources
# ----------------------------------------------------------
import logging
from threading import Timer
from datetime import datetime
from ...kernel.agent.Action import Action

# ----------------------------------------------------------
# Define the action
# ----------------------------------------------------------
class TimeoutAction(Action):
    """
    @See Action
    """

    def __init__(self):
        self.__timer = None
        super().__init__()

    def handler(self):
        """ Timeout handler """
        if self.agent.isTimeout():
            self.agent.setTimeout(False)
            self.adm.sendEvent(self.agent.id, 'response', 'timeout')

    def execute(self, data):
        """
        @param data
        """
        if data['command'] == 'start':
            if not self.agent.isTimeout():
                self.agent.setTimeout(True)
                self.__timer = Timer(data['time'], self.handler)
                self.__timer.start()
        else:
            if self.__timer:
                self.__timer.cancel()
                self.__timer = None

    def catchException(self, exception):
        """
        Catch the exception.
        @param exception Response exception
        """
        pass



from abc import ABC, abstractmethod

class World(ABC):
	
	def __init__(self):
		self.agent = None
		super().__init__()
	
	@abstractmethod
	def update(self, event, data):
		pass

	def setAgent(self, agent):
		self.agent = agent



from abc import abstractmethod
from ...kernel.agent.Action import Action

# --------------------------------------------------------
# Define Action
# --------------------------------------------------------
class ResponseAction(Action):
    """ An action is a response to the occurrence of an event """
    
    def sendResponse(self, request):
        if len(request['dtoList']) == 1:
            request['gateway'].put(request['dtoList'][0])
        else:
            request['gateway'].put(request['dtoList'])

    def execute(self, data):
        """ 
        Response.
        @param data Event data 
        """
        request = self.agent.getRequestDict()[data['source']]
        if 'timeout' in data:
            self.sendResponse(request)
        else:
            request['dtoList'].append(data['result'])
            if len(request['dtoList']) >= self.agent.getBufferSize():
                self.sendResponse(request)
        
    def catchException(self, exception):
        """
        Catch the exception.
        @param exception Response exception
        """
        pass



from abc import ABC, abstractmethod

class Goal(ABC):

    id = None
    plan = None
    execute = None
    priority = None
    settings = None
    
    def __init__(self):
        self.execute = False
        self.settings = self.setUp()
        self.id = self.settings['id']
        self.plan = self.settings['plan']
        self.priority = self.settings['priority']
        super().__init__()

    @abstractmethod
    def setUp(self):
        pass

    @abstractmethod
    def eval(self, believes):
        pass






from threading import Thread
from ...kernel.util.Queue import Queue
from .GoalExe import GoalExe
from .BDILevel import BDILevel

import time
import random

class BDIMachine(Thread):

    let = None
    alive = None
    agent = None
    queue = None
    planQueue = None

    executeList = None

    machine = None
    physiologyList = None
    safetyList = None
    socialList = None
    esteemList = None
    selfrealizationList = None

    def __init__(self, agent):
        self.let = True
        self.alive = True
        self.agent = agent
        self.queue = agent.queue

        self.executeList = []

        self.physiologyList = []
        self.safetyList = []
        self.socialList = []
        self.esteemList = []
        self.selfrealizationList = []

        goals = agent.settings['goals']
        for goa in goals:
            if goa.priority == BDILevel.PHYSIOLOGY:
                self.physiologyList.append(goa)
            if goa.priority == BDILevel.SAFETY:
                self.safetyList.append(goa)
            if goa.priority == BDILevel.SOCIAL:
                self.socialList.append(goa)
            if goa.priority == BDILevel.ESTEEM:
                self.esteemList.append(goa)
            if goa.priority == BDILevel.SELF_REALIZATION:
                self.selfrealizationList.append(goa)

        self.pool = Queue(10)
        self.planQueue = Queue(10)

        for x in range(1,5):
            ge = GoalExe(self.pool, self.planQueue)
            ge.start()
            self.pool.put(ge)

        super().__init__()

    def run(self):        
        while self.alive:
            if self.let:
                evt = self.queue.get()
                self.queue.task_done()

                # Update World
                self.agent.world.update(evt['event'], evt['data']) 

                # Run Machine                                
                if self.checkFire(self.physiologyList):
                    self.executeGoals()
                else:
                    if self.checkFire(self.safetyList):
                        self.executeGoals()
                    else:
                        if self.checkFire(self.socialList):
                            self.executeGoals()
                        else:
                            if self.checkFire(self.esteemList):
                                self.executeGoals()
                            else:
                                if self.checkFire(self.selfrealizationList):
                                    self.executeGoals()


    def executeGoals(self):
        for goal in self.executeList:
            if not goal.execute:
                self.pool.get()
                self.pool.task_done()
                goal.execute = True
                self.executeList.remove(goal)
                self.planQueue.put(goal)                
                
    def contains(self, goal, lst):
        for go in lst:
            if go.id == goal.id:
                return True
        return False

    def checkFire(self, lst):
        for goa in lst:
            if not goa.execute and goa.eval(self.agent.believes):
                if not self.contains(goa, self.executeList):
                    self.executeList.append(goa)
        return ( len(self.executeList) > 0 )

    def setLet(self, val):
        self.let = val

    def setAlive(self, val):
        self.alive = val




import os
import json
import socket
import socketserver
from time import sleep
from importlib.machinery import SourceFileLoader
from ...kernel.system.Directory import Directory
        
class RemoteAdmHandler(socketserver.StreamRequestHandler):

    def handle(self):
        self.data = self.rfile.readline().strip()
        print("{} wrote:".format(self.client_address[0]))
        print(self.data)
        msg = str(self.data, "utf-8")
        info = json.loads(msg)

        if info['command'] == 'REGISTER':
            directory = Directory()
            directory.addContainer({'name': info['name'], 'host': info['host'], 'port': info['port']})
            data = 'ACK'
            self.wfile.write(bytes(data + "\n", "utf-8"))
            sleep(2) # 2 seconds.
            containers = directory.getContainers()
            agents = directory.getAgents()
            dto = '{"command":"UPDATE", "agents" : ' + json.dumps(agents, ensure_ascii=False) + '}'
            for ctn in containers:
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                try:                            
                    sock.connect( (ctn['host'], int(ctn['port'])) )                            
                    sock.sendall(bytes(dto + "\n", "utf-8"))
                except:
                    pass
                    
                finally:
                    sock.close()
            directory.checkFull(info['name'])

        if info['command'] == 'MOVE':
            appPath = os.path.abspath(os.path.dirname(sys.argv[0]))
            appPathSplit = appPath.split(os.sep)
            classPath = info['path']
            classPathSplit = classPath.split('.')
            path = appPathSplit[0]
            findFlag = False
            for x in range(1, len(appPathSplit) ):
                if appPathSplit[x] == classPathSplit[0]:
                    path = path + os.sep + appPathSplit[x]
                    for y in range(1, len(classPathSplit) ):
                        path = path + os.sep + classPathSplit[y]
                    break
                else:
                    path = path + os.sep + appPathSplit[x]            
            path = path + '.py'
            module = SourceFileLoader(info['class'], path).load_module()
            agType = getattr(module, info['class'])
            ag = agType(info['id'])
            ag.state = info['state']
            ag.start()

        """    
        if info['command'] == 'UPDATE':        
            agents = info['agents']
            directory = Directory()
            for agent in agents:
                directory.resetAgentList()
                directory.addAgent(agent)
            containers = directory.getContainers()
            agents = directory.getAgents()
            dto = '{"command":"UPDATE", "agents" : ' + json.dumps(agents, ensure_ascii=False) + '}'
            for ctn in containers:
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                try:                            
                    sock.connect( (ctn['ip'], int(ctn['port'])) )                            
                    sock.sendall(bytes(dto + "\n", "utf-8"))
                except:
                    pass
                    
                finally:
                    sock.close()
        """
        if info['command'] == 'ADD':
            agent = info['agent']
            host = agent['host']
            directory = Directory()
            directory.addAgent(agent)    
            containers = directory.getContainers()
            agents = directory.getAgents()
            dto = '{"command":"UPDATE", "agents" : ' + json.dumps(agents, ensure_ascii=False) + '}'
            for ctn in containers:
                if not ctn['host'] == host:
                    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    try:                            
                        sock.connect( (ctn['host'], int(ctn['port'])) )                            
                        sock.sendall(bytes(dto + "\n", "utf-8"))
                    except:
                        pass
                    finally:
                        sock.close()
        
        if info['command'] == 'SENDEVENT':
            from ...kernel.system.Adm import Adm
            Adm().sendEvent(info['id'], info['event'], info['data'])
            rsp = 'ACK'
            self.wfile.write(bytes(rsp + "\n", "utf-8"))



from .task import Task
from abc import abstractmethod
from .exceptions import WorkerException
from ...kernel.agent.Agent import Agent
from .timeoutaction import TimeoutAction

class Worker(Agent):

    def __init__(self, agentID):
        self.__taskList = []
        self.__controller = None
        self.__controllerType = None    
        super().__init__(agentID)

    def setUp(self):
        self.addBehavior('Task')
        self.addBehavior('Timeout')
        self.bindAction('Timeout', 'timeout', TimeoutAction())
        self.build()

    def bindTask(self, action):
        if isinstance(action, Task):
            self.__taskList.append(action)    
            self.bindAction('Task', 'task', action)
        else:
            raise WorkerException('[Warn, bindTask]: The action must inherit from the task type')

    def suscribeRemoteController(self, controllerID):
        self.setController(controllerID)
        self.setControllerType('LINEAL')
        actions = self.getActions()
        for action in actions:
            action.setIsPool(False)
            action.setEnableResponse(True)

    @abstractmethod
    def build(self):
        pass

    def getActions(self):
        return self.__taskList

    def getController(self):
        return self.__controller

    def setController(self, controller):
        self.__controller = controller

    def setControllerType(self, controllerType):
        self.__controllerType = controllerType



from abc import ABC, abstractmethod
from ...kernel.util.Queue import Queue
from ...kernel.system.Adm import Adm
from .BDIMachine import BDIMachine 

class BDIAg(ABC):

    id = None    
    queue = None     
    world = None
    goals = None
    believes = None
    settings = None

    def __init__(self):
        self.settings = self.setUp()
        self.id = self.settings['id']		
        self.world = self.settings['world']
        self.goals = self.settings['goals']
        self.believes = self.settings['believes']
        Adm().addAgent(self)

        self.queue = Queue(10)
        self.machine = BDIMachine(self)

        self.world.setAgent(self)

        for goal in self.goals:
            for act in goal.plan:
                act.setAgent(self)

        super().__init__()

    def sendEvent(self, event, data):
        self.queue.put({'event': event, 'data': data})

    def start(self):
        self.machine.start()

    @abstractmethod
    def setUp(self):
        pass



from .exceptions import SystemException

class Directory(object):
    class __Directory:
        
        def __init__(self):
            self.containerList = []
            self.agentList = []
            self.gateway = None
            self.checkToList = None
                    
        def addContainer(self, container):
            self.containerList.append(container)

        def addAgent(self, agent):
            self.agentList.append(agent)

        def getContainers(self):
        	return self.containerList

        def getAgents(self):
        	return self.agentList
        
        def setAgentList(self, agentList):
            self.agentList = agentList
            
        def getAgent(self, agentID):
            for agent in self.agentList:
                if agentID == agent['agent']:
        	        return agent
            return None

        def resetAgentList(self):
        	self.agentList = []

        def removeAgent(self, agent): 
            for ag in self.agentList:
                if ag['agent'] == agent:
                    self.agentList.remove(ag)

        def setCheckList(self, gateway, checkToList):
            self.gateway = gateway
            self.checkToList = checkToList
        
        def checkFull(self, containerName):
            if self.checkToList:
                if containerName in self.checkToList:
                    self.checkToList.remove(containerName)
                    if len(self.checkToList) == 0:
                        self.gateway.put(None)
                else:
                    raise SystemException('Container name: "%s" does not match' % containerName)
                    
    instance = None
    def __new__(cls):
        if not Directory.instance:
            Directory.instance = Directory.__Directory()
        return Directory.instance
    def __getattr__(self, name):
        return getattr(self.instance, name)
    def __setattr__(self, name):
        return setattr(self.instance, name)



# -*- coding: utf-8 -*-
"""
----------------------------------------------------------
------------------------- PBESA --------------------------
----------------------------------------------------------

@autor AKEN & SIDRE
@version 3.0.1
@date 27/07/20
"""

# --------------------------------------------------------
# Define resources
# --------------------------------------------------------
import time
import random
import traceback
from threading import Thread
from ...kernel.util.Queue import Queue
from ...kernel.agent.exceptions import ActionException

# --------------------------------------------------------
# Define component
# --------------------------------------------------------
class BehaviorExe(Thread):
    """ Behavior executor component """

    def __init__(self, queue):
        self.__let = False
        self.__alive = True
        self.__stopAgent = Queue(1)
        self.__queue = queue
        super().__init__()

    def run(self):
        while self.__alive:
            evt = self.__queue.get()
            if not self.__let:
                self.__stopAgent.get()
            try:
                if self.__alive:
                    evt['action'].execute(evt['data'])
            except Exception as e:
                traceback.print_exc()
                evt['action'].catchException(e)
                self.__let = False        
            self.__queue.task_done()
        
    def setLet(self, val):
        self.__let = val
       
    def notifyLet(self, val):
        self.__stopAgent.put(None)
        
    def setAlive(self, val):
        self.__alive = val

    def finalize(self):
        self.__let = False
        self.__queue.put(None)
        self.__stopAgent.put(None)



from threading import Thread
#from ...kernel.system.Adm import Adm
from ...kernel.adapter.Adapter import Adapter
from ...middleware.adapter.WSSNJHandler import WSSNJHandler
from ...middleware.adapter.SubProcessAdapter import SubProcessAdapter
from ...kernel.io.TCPServer import TCPServer

import os
import sys
import json
#import socketserver

import threading

class WSSNJAdapter(Adapter, Thread):

    IP = None
    adm = None
    PORT = None
    server = None

    def __init__(self):
        self.adm = Adm()
        conf = self.adm.getConf()
        self.IP = conf['ip']
        self.PORT = conf['port']
        self.INTERNAL_PORT = conf['internal_port']
        super().__init__()

    def setUp(self):
        DIR = os.path.dirname(os.path.abspath(__file__)) 
        FILE = os.path.join(DIR,"WSSNJWrapper.js")
        spa = SubProcessAdapter("node", FILE, self.IP + '-' + str(self.PORT) + '-' + str(self.INTERNAL_PORT))
        spa.start()
        
    def response(self, response):
        pass
    
    def request(self):
        pass

    def finalize(self):
        self.server.shutdown()
        self.server.server_close()        

    def run(self):
        self.server = TCPServerTK((self.IP, self.INTERNAL_PORT), WSSNJHandlerTK)
        server_thread = threading.Thread(target=self.server.serve_forever)
        server_thread.daemon = True
        server_thread.start()




from abc import abstractmethod
from ...kernel.util.Queue import Queue
from .exceptions import LinealException
from ...kernel.agent.Agent import Agent
from .timeoutaction import TimeoutAction
from .delegateaction import DelegateAction
from .responseaction import ResponseAction

class LinealController(Agent):
    
    def __init__(self, agentID):
        self.__agentList = []
        self.__checkDict = {}
        self.__gateway = None
        self.__freeList = None
        self.__timeout = False
        self.__poolSize = None
        self.__requestDict = {}    
        super().__init__(agentID)

    def setUp(self):
        self._social = True
        self.addBehavior('Timeout')
        self.bindAction('Timeout', 'timeout', TimeoutAction())
        self.addBehavior('Delegate')
        self.addBehavior('Response')
        self.build()

    def bindDelegateAction(self, action):
        self.bindAction('Delegate', 'delegate', action)

    def bindResponseAction(self, action):
        self.bindAction('Response', 'response', action)

    def suscribeAgent(self, agent):
        if not isinstance(agent, Agent):
            raise LinealException('[Warn, suscribeAgent]: The object to subscribe is not an agent')
        agent.setController(self.id)
        agent.setControllerType('LINEAL')
        self.__checkDict[agent.id] = None
        self.__agentList.append(agent.id)
        actions = agent.getActions()
        for action in actions:
            action.setIsPool(False)
            action.setEnableResponse(True)
    
    def suscribeRemoteAgent(self, agentID):
        if not isinstance(agentID, str):
            raise LinealException('[Warn, suscribeRemoteAgent]: The object to subscribe is not an agent ID')
        self.__checkDict[agentID] = None
        self.__agentList.append(agentID)
        
    def reset(self):
        self.__freeList = []
        for ag in self.__agentList:
            self.__checkDict[ag] = None
            self.__freeList.append(ag)

    @abstractmethod
    def build(self):
        pass

    def start(self):
        if len(self.__agentList) <= 0:
            raise LinealException('[Warn, toAssign]: Controller agent list is empty. Agents must be subscribed to the controller')
        super().start()

    def getFreeList(self):
        return self.__freeList

    def getRequestDict(self):
        return self.__requestDict

    def getGateway(self):
        return self.__gateway

    def setGateway(self, gateway):
        self.__gateway = gateway

    def getCheckDict(self):
        return self.__checkDict
    
    def isTimeout(self):
        return self.__timeout

    def setTimeout(self, timeout):
        self.__timeout = timeout

    def isBlock(self):
        return True



import json
import socket
import socketserver
from time import sleep
#from ...kernel.system.Adm import Adm
from ...kernel.system.Directory import Directory

class WSSNJHandler(socketserver.BaseRequestHandler):

    def handle(self):
        try:
            while True:
                #data = self.rfile.readline().strip()
                data = self.request.recv(1024)
                print("[REQUEST]: {}".format(self.client_address[0]))
                print(data)
                msg = str(data, "utf-8")
                rqst = json.loads(msg)                
                dto = {'socket': self.request, 'request': rqst}
                Adm().sendEvent('front_controller', 'wsnj_delegate', dto)
        except Exception as inst:
            print('Controled ecxeption in WSSNJHandler: ')
            print(inst)



from threading import Thread
#from ...kernel.SystemTK.Adm import Adm
from ...kernel.util.Queue import Queue
from ...kernel.adapter.Adapter import Adapter

import json
import asyncio
#import websockets

class WSSAdapter(Adapter, Thread):

    IP = None
    adm = None
    PORT = None
    server = None

    def __init__(self):
        self.adm = Adm()
        conf = self.adm.getConf()
        self.IP = conf['ip']
        self.PORT = conf['port']
        super().__init__()

    def setUp(self):
        pass
        
    def response(self, response):
        pass
    
    def request(self):
        pass

    def finalize(self):
        pass

    def run(self):
        asyncio.set_event_loop(asyncio.new_event_loop())
        start_server = websockets.serve(self.handler, self.IP, self.PORT)
        asyncio.get_event_loop().run_until_complete(start_server)
        asyncio.get_event_loop().run_forever()

    # async def handler(self, websocket, path):
    #     queue = Queue(10)
    #     consumer_task = asyncio.ensure_future(self.requestHandler(websocket, path, queue))
    #     producer_task = asyncio.ensure_future(self.responseHandler(websocket, path, queue))
    #     done, pending = await asyncio.wait(
    #         [consumer_task, producer_task],
    #         return_when=asyncio.FIRST_COMPLETED,
    #     )
    #     for task in pending:
    #         task.cancel()

    async def requestHandler(self, websocket, path, queue):
        while True:
            print(queue)
            async for message in websocket:
                request = json.loads(message)
                dto = {'socket': queue, 'request': request}
                self.adm.sendEvent('front_controller', 'ws_request', dto)

    async def responseHandler(self, websocket, path, queue):
        while True:
            message = await queue.get()
            await websocket.send(message)

    async def handler(self, websocket, path):
        queue = Queue(10)
        try:
            while True:
                request = json.loads(await websocket.recv())
                print ('REQUEST: ', request)
                dto = {'socket': queue, 'request': request}
                self.adm.sendEvent('front_controller', 'ws_request', dto)
                response = str(queue.get())
                response = response.replace("'", "\"")
                print ('RESPONSE: ', response)
                await websocket.send(response)
        except Exception as inst:
            print('Controled ecxeption: ')
            print(inst)






from threading import Thread
import time
import random

class GoalExe(Thread):

    let = None
    pool = None
    alive = None
    goal = None
    planQueue = None

    def __init__(self, pool, planQueue):
        self.let = True
        self.pool = pool
        self.alive = True
        self.planQueue = planQueue
        super().__init__()

    def run(self):
        while self.alive:
            if self.let:
                goal = self.planQueue.get()
                self.planQueue.task_done()
                for action in goal.plan:
                    action.execute('')
                goal.execute = False    
                self.pool.put(self)                
            #time.sleep(random.random())
    
    def setLet(self, val):
        self.let = val

    def setAlive(self, val):
        self.alive = val




# -*- coding: utf-8 -*-
"""
----------------------------------------------------------
------------------------- PBESA --------------------------
----------------------------------------------------------

@autor AKEN & SIDRE
@version 3.0.1
@date 27/07/20
"""

# --------------------------------------------------------
# Define resources
# --------------------------------------------------------
import logging
from abc import ABC, abstractmethod
from ...kernel.system.Adm import Adm
from ...kernel.util.Queue import Queue
from ...kernel.agent.Channel import Channel
from ...kernel.agent.BehaviorExe import BehaviorExe
from ...kernel.agent.exceptions import AgentException

# --------------------------------------------------------
# Define component
# --------------------------------------------------------
class Agent(ABC):
    """ Represents a system agent """
    
    def __init__(self, agentID):
        """
        Agent constructor method.
        @param agentID Unic agent ID
        """
        if agentID and isinstance(agentID, str):    
            self.id = agentID
            self.state = {}
            self.__eventsTable = {}
            self.__channelsTable = {}
            self.__workerList = []
            self.__channelList = []
            self.__behaviors = {}
            self.__behaviorsTable = None
            self._social = False
            self.log = None
            self.__buildAgent()
            Adm().addAgent(self)
            super().__init__()
        else:
            raise AgentException('[Fatal, __init__]: The agent ID must be a str')
    
    def __buildAgent(self):
        """ Build the agent structure """
        self.setUp()
        if len(self.__behaviors) > 0: 
            for key, beh in self.__behaviors.items():            
                queue = Queue(10)
                channel = Channel(queue)    
                worker = BehaviorExe(queue)
                self.__channelsTable[key] = {'channel' : channel, 'worker': worker}  
                self.__workerList.append(worker)
                self.__channelList.append(channel)
                for evts in beh:
                    try:
                        evts['action'].setAgent(self)
                        self.__eventsTable[evts['event']] = {'behavior' : key, 'action': evts['action']}
                    except:
                        raise AgentException('[Fatal, buildAgent]: The action must be instantiated: %s' % str(evts['action']))            
        else:
            raise AgentException('[Fatal, buildAgent]: Agent behaviors must be defined')
     
    @abstractmethod
    def setUp(self):
        """ Method to create and initialize the agent structure """
        pass

    @abstractmethod
    def shutdown(self):
        """ Method to free up the resources taken by the agent """
        pass

    def sendEvent(self, event, data):
        """
        Method that registers an event to the agent.
        @param event Envent
        @param Data event
        @exceptions AgentException
        """
        if event in self.__eventsTable:    
            behavior = self.__eventsTable[event]
            channel = self.__channelsTable[behavior['behavior']]
            evt = {'event': event, 'data': data, 'action': behavior['action']}              
            channel['channel'].sendEvent(evt)
        else:
            raise AgentException('[Warn, sendEvent]: The agent has not registered the event %s' % event)
    
    def start(self):
        for w in self.__workerList:
            w.setLet(True)
            w.start()
            
    def wait(self):
        for w in self.__workerList:
            w.setLet(False)

    def finalize(self):
        for w in self.__workerList:
            w.setAlive(False)
            w.finalize()
    
    def kill(self):
        """ Remove the agent from the system """
        self.shutdown()
        self.id = None
        self.log = None
        self.state = None
        self.__eventsTable = None
        self.__channelsTable = None
        self.finalize()
        self.__workerList = None
        self.__channelList = None
        self.__behaviors = None

    def toDTO(self):
        dto = {
            'command': 'MOVE',
            'class': self.__class__.__name__,
            'path': self.__module__,
            'id': self.id,
            'state': self.state  
        }
        rtn = str(dto)
        rtn = rtn.replace("'", "\"")  
        return rtn

    def addBehavior(self, behavior):
        """
        Add the new behavior to the agent's behavior.
        @param behavior New behavior
        """
        self.__behaviors[behavior] = []

    def bindAction(self, behavior, event, action):
        """
        Link behavior to event with action.
        @param behavior Behavior
        @param event Event link to behavior
        @param action Action link to event
        @exceptions AgentException
        """
        if behavior in self.__behaviors:
            self.__behaviors[behavior].append({
                'event': event, 
                'action': action
            })
        else:
            raise AgentException('[Fatal, bindAction]: The behavior "%s" is not associated with the agent. Must be added before behavior' % behavior)

    def setUpLogger(self, loggerName, loggerFile, level):
        """
        Inicia un componente de seguimiento de la aplicacion.
        @param loggerName nombre del log
        @param loggerFile ruta del archivo
        """
        l = logging.getLogger(loggerName)
        formatter = logging.Formatter('[PBESA]: %(asctime)s %(name)-12s %(lineno)d %(levelname)-8s %(message)s')
        fileHandler = logging.FileHandler(loggerFile, 'w', 'utf-8')
        fileHandler.setFormatter(formatter)
        streamHandler = logging.StreamHandler()
        streamHandler.setFormatter(formatter)
        l.setLevel(level)
        l.addHandler(fileHandler)
        l.addHandler(streamHandler)

    def activeLogger(self, logger, level=logging.INFO):
        if not level:
            level = logging.INFO
        self.setUpLogger(logger, '%s.log' % logger, level)
        self.log = logging.getLogger(logger)
    
    def suscribeLogger(self, logger):
        self.log = logging.getLogger(logger)

    def isSocial(self):
        return self._social




from abc import abstractmethod
from ...kernel.agent.Action import Action

# --------------------------------------------------------
# Define Action
# --------------------------------------------------------
class NotifyFreeAction(Action):
    """ An action is a response to the occurrence of an event """
    
    def execute(self, data):
        """ 
        Response.
        @param data Event data 
        """
        self.agent.getFreeQueue().put(data)
        
    def catchException(self, exception):
        """
        Catch the exception.
        @param exception Response exception
        """
        pass



import traceback
import socketserver
from threading import Thread
from .exceptions import RemoteException
from ...middleware.remote.RemoteAdmHandler import RemoteAdmHandler

class RemoteAdm(Thread):

    __IP = None
    __PORT = None

    def __init__(self, ip, port):
        self.__IP = ip
        self.__PORT = port
        super().__init__()

    def run(self):
        try:        
            server = socketserver.TCPServer((self.__IP, self.__PORT), RemoteAdmHandler)
            server.serve_forever()
        except:
            traceback.print_exc()
            raise RemoteException("Could not initialize master container on %s host and %d port" % (self.__IP, self.__PORT))



import json

class SystemFile(object):
    
    path = None

    def __init__(self, path):
        self.path = path
    
    def readJsonFile(self):
        with open(self.path) as f:
            data = json.load(f)
        return data



# -*- coding: utf-8 -*-
"""
----------------------------------------------------------
------------------------- PBESA --------------------------
----------------------------------------------------------

@autor AKEN & SIDRE
@version 3.0.1
@date 27/07/20
"""

# --------------------------------------------------------
# Define resources
# --------------------------------------------------------
import os
import gc
import sys
import json
import socket
import traceback
from time import sleep
from ...kernel.util.Queue import Queue
from ...kernel.system.Directory import Directory
from ...middleware.remote.RemoteAdm import RemoteAdm
from ...kernel.adapter.FileAdapter import FileAdapter
from ...kernel.system.exceptions import SystemException
from ...middleware.remote.AdmListener import AdmListener

# --------------------------------------------------------
# Define component
# --------------------------------------------------------
class Adm(object):
    """ Represents the agent container manager """

    class __Adm:
        """ Singleton design pattern """

        def __init__(self):
            """ Default constructor """
            self.val = None
            self.conf = None
            self.adapters = {}
            self.agentsTable = {}
            self.containerList = []
            
        def __str__(self):
            """ To string """
            return repr(self) + self.val

        def start(self):
            """ Default administrator startup """
            self.conf = {
                "user" : "local",
                "host" : "localhost",
                "port" : 8080,
                "remote" : None     
            }

        def startByConf(self, conf):            
            """
            Administrator startup by configuration.
            @param conf configuration dictionary or 
                   configuration file path
            @exceptions SystemException
            """
            if conf:
                if not isinstance(conf, str):
                    if 'user' in conf and 'host' in conf and 'port' in conf:
                        if conf['user'] and conf['host'] and conf['port']:
                            self.conf = conf
                        else:
                            raise SystemException('[Warn, startByConf]: Configuration parameters cannot be null')        
                    else:
                        raise SystemException('[Warn, startByConf]: A parameter is missing. The parameters are: {user, host, port, remote(optional)}')
                else:
                    CONF_DIR = conf
                    fa = FileAdapter({'alias':'JsonAdapter', 'type': 'JSON', 'path': CONF_DIR})
                    fa.setUp()
                    param = fa.request()
                    self.conf = param['conf']
                if 'remote' in self.conf and self.conf['remote']:
                    remote = self.conf['remote']                
                    if remote['master_mode']:
                        self.remoteAdm = RemoteAdm(self.conf['host'], self.conf['port'])
                        self.remoteAdm.start()
                    else:
                        if 'container_name' in self.conf and 'master_host' in remote and 'master_port' in remote:
                            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)                        
                            attempts = remote['attempts']
                            data = '{"command": "REGISTER", "name":"' + self.conf['container_name'] + '", "host":"' + self.conf['host'] + '", "port":"'+ str(self.conf['port']) +'"}'
                            received = None
                            for x in range(1, attempts):                        
                                try:
                                    sock.connect((remote['master_host'], remote['master_port']))                            
                                    sock.sendall(bytes(data + "\n", "utf-8"))
                                    received = str(sock.recv(1024), "utf-8")                                                           
                                    if received == 'ACK' + "\n":
                                        break
                                except:
                                    sleep(1)
                                finally:
                                    sock.close()
                            if received == 'ACK' + "\n":
                                admListener = AdmListener(self.conf['host'], self.conf['port'])
                                admListener.start()
                            else:
                                raise SystemException('[Warn, startByConf]: The administrator could not connect with the administrator master') 
                        else:
                            raise SystemException('[Warn, startByConf]: A parameter is missing. The parameters are: {container_name, master_host, master_port}')
            else:
                raise SystemException('[Warn, startByConf]: The parameter "conf" cannot be NoneType')

        def killAgent(self, agent):
            """
            Remove an agent of the system.
            @param agent Agent to add
            @exceptions SystemException  
            """
            del self.agentsTable[agent.id]
            directory = Directory()
            directory.removeAgent(agent.id)
            agent.kill()
            if self.conf['remote']:
                remote = self.conf['remote']
                agents = directory.getAgents()
                dto = '{"command":"UPDATE", "agents" : ' + json.dumps(agents, ensure_ascii=False) + '}'                
                if remote['mode'] == 'MASTER':
                    containers = directory.getContainers()                    
                    for ctn in containers:
                        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                        try:                            
                            sock.connect( (ctn['ip'], int(ctn['port'])) )                            
                            sock.sendall(bytes(dto + "\n", "utf-8"))
                        except:
                            raise SystemException('[Warn, killAgent]: Could not update container with IP %s' % ctn['ip'])                    
                        finally:
                            sock.close()
                else:
                    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    master = remote['master']                                              
                    try:                            
                        sock.connect((master['master_ip'], master['master_port']))                            
                        sock.sendall(bytes(dto + "\n", "utf-8"))
                    except:
                        raise SystemException('[Warn, killAgent]: Could not update container master')
                    finally:
                        sock.close()
            agent = None
            gc.collect()

        def addAgent(self, agent):
            """
            Add an agent to the system.
            @param agent Agent to add
            @exceptions SystemException  
            """
            if agent.id in self.agentsTable:
                raise SystemException('[Warn, addAgent]: Agent ID: "%s" already exists in container' % agent.id)
            else:
                self.agentsTable[agent.id] = agent
                host = self.conf['host']
                port = self.conf['port']
                if 'remote' in self.conf and self.conf['remote']:
                    remote = self.conf['remote'] 
                    #host = remote['host']
                    #port = remote['port']
                directory = Directory()
                agentDTO = {'agent': agent.id, 'host': host, 'port' : port}
                directory.addAgent(agentDTO)
                if 'remote' in self.conf and self.conf['remote']:
                    remote = self.conf['remote']
                    if remote['master_mode']:
                        agents = directory.getAgents()
                        dto = '{"command":"UPDATE", "agents": ' + json.dumps(agents, ensure_ascii=False) + '}'
                        containers = directory.getContainers()                    
                        for ctn in containers:
                            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                            try:                            
                                sock.connect( (ctn['host'], int(ctn['port'])) )                            
                                sock.sendall(bytes(dto + "\n", "utf-8"))
                            except:
                                raise SystemException('[Warn, addAgent]: Could not update container with %s host' % ctn['host'])                    
                            finally:
                                sock.close()
                    else:
                        dto = '{"command":"ADD", "agent": ' + json.dumps(agentDTO, ensure_ascii=False) + '}'
                        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)                                             
                        try:                            
                            sock.connect((remote['master_host'], remote['master_port']))                            
                            sock.sendall(bytes(dto + "\n", "utf-8"))
                        except:
                            raise SystemException('[Warn, addAgent]: Could not update container master')
                        finally:
                            sock.close()
        
        def sendEvent(self, agentID, event, data):
            """
            Send an event to another agent.
            @param agentID Destination agent ID
            @param event Event to send
            @param data Event data
            @return :bool True if the submission was 
                    successful | False otherwise
            @exceptions SystemException
            """
            if agentID in self.agentsTable:
                ag = self.agentsTable[agentID]
                ag.sendEvent(event, data)
                return True
            else:
                if 'remote' in self.conf and self.conf['remote']:
                    remote = self.conf['remote'] 
                    attempts = remote['attempts']
                    agent = Directory().getAgent(agentID)
                    received = None
                    if agent:
                        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                        aux = str(data)
                        aux = aux.replace("\'", "\"")                                                
                        dto = '{"command": "SENDEVENT", "id":"' + agentID + '", "event":"' + event + '", "data":'+ aux +'}' 
                        for it in range(1, attempts):                        
                            try:
                                print('attempt: %d' % it)                            
                                sock.connect((agent['host'], agent['port']))                            
                                sock.sendall(bytes(dto + "\n", "utf-8"))
                                received = str(sock.recv(1024), "utf-8")                                                            
                                if received == 'ACK' + "\n":
                                    break
                            except:
                                traceback.print_exc()
                                sleep(1)
                            finally:
                                sock.close()
                    if received == 'ACK' + "\n":
                        return True
                    else:
                        raise SystemException('[Warn, sendEvent]: The event could not be sent to the agent with the ID: %s ' % agentID)                     
                else:
                    raise SystemException('[Warn, sendEvent]: An agent with the ID %s could not be found in the system' % agentID)

        def addAdapter(self, id, adapter):
            self.adapters[id] = adapter

        def getAdapter(self, id):
            return self.adapters[id]

        def getConf(self):
            return self.conf

        def addContainer(self, container):
            self.containerList.append(container)

        def moveAgent(self, agentID, container):
            """
            Add an agent to the system.
            @param agent Agent to add
            @exceptions SystemException  
            """
            ag = self.agentsTable[agentID]
            ag.finalize()
            dto = ag.toDTO()
            print (dto)
            containers = Directory().getContainers()
            for ctn in containers:
                if container == ctn['name']:
                    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    try:                            
                        sock.connect( (ctn['host'], int(ctn['port'])) )                            
                        sock.sendall(bytes(dto + "\n", "utf-8"))
                    except:
                        raise SystemException('[Error, moveAgent]: Could not update container with IP %s' % ctn['host'])
                    finally:
                        sock.close()
                    break    
            self.agentsTable[agentID] = None
            Directory().removeAgent(agentID)

        def setEndPoint(self, agID, socket):
            ag = self.agentsTable[agID]
            ag.setSocket(socket)

        def callAgent(self, agentID, data):
            """
            Call a social agent.
            @param data Call data
            """
            if agentID in self.agentsTable:
                ag = self.agentsTable[agentID]
                if ag.isSocial():
                    if ag.isBlock():
                        queue = Queue(1)
                        dto = {
                            'dto': data,
                            'gateway': queue
                        }
                        ag.sendEvent('delegate', dto)
                        result = queue.get()
                        queue.task_done()
                        return result
                    else:
                        raise SystemException('[Warn, callAgent]: The "callAgent" method is only for blocking controllers')
                else:
                    raise SystemException('[Warn, callAgent]: Only social agents can be invoked in this method')
            else:
                raise SystemException('[Warn, callAgent]: An agent with the ID %s could not be found in the system' % agentID) 

        def submitAgent(self, agentID, data):
            """
            Call a social agent.
            @param data Call data
            """
            if agentID in self.agentsTable:
                ag = self.agentsTable[agentID]
                if ag.isSocial():
                    if not ag.isBlock():
                        ag.sendEvent('delegate', data)
                    else:
                        raise SystemException('[Warn, submitAgent]: The "submitAgent" method is only for non-blocking controllers')
                else:
                    raise SystemException('[Warn, submitAgent]: Only social agents can be invoked in this method')
            else:
                raise SystemException('[Warn, submitAgent]: An agent with the ID %s could not be found in the system' % agentID) 

        def waitFull(self, containerList):
            gateway = Queue(1)
            Directory().setCheckList(gateway, containerList)
            gateway.get()
            gateway.task_done()

        def destroy(self):
            for agent in self.agentsTable:
                self.killAgent(agent)
            self.val = None
            self.conf = None
            self.adapters = None
            self.agentsTable = None
            self.containerList = None
            gc.collect()        
    
    # ----------------------------------------------------
    # Defines singleton instance
    # ----------------------------------------------------

    instance = None
    def __new__(cls):
        if not Adm.instance:
            Adm.instance = Adm.__Adm()
        return Adm.instance
    def __getattr__(self, name):
        return getattr(self.instance, name)
    def __setattr__(self, name):
        return setattr(self.instance, name)



import os
import sys
import json
import inspect
import socketserver
from time import sleep
from importlib.machinery import SourceFileLoader
from ...kernel.system.Directory import Directory

import types

class AdmListenerHandler(socketserver.StreamRequestHandler):

    def handle(self):
        self.data = self.rfile.readline().strip()
        print("{} wrote:".format(self.client_address[0]))
        print(self.data)
        msg = str(self.data, "utf-8")
        info = json.loads(msg)    
        if info['command'] == 'UPDATE':        
            agents = info['agents']
            directory = Directory()
            directory.setAgentList(agents)
        if info['command'] == 'MOVE':   
            appPath = os.path.abspath(os.path.dirname(sys.argv[0]))
            appPathSplit = appPath.split(os.sep)
            classPath = info['path']
            classPathSplit = classPath.split('.')
            path = appPathSplit[0]
            findFlag = False
            for x in range(1, len(appPathSplit) ):
                if appPathSplit[x] == classPathSplit[0]:
                    path = path + os.sep + appPathSplit[x]
                    for y in range(1, len(classPathSplit) ):
                        path = path + os.sep + classPathSplit[y]
                    break
                else:
                    path = path + os.sep + appPathSplit[x]            
            path = path + '.py'
            module = SourceFileLoader(info['class'], path).load_module()
            agType = getattr(module, info['class'])
            ag = agType(info['id'])
            ag.state = info['state']
            ag.start()
        if info['command'] == 'SENDEVENT':
            from ...kernel.system.Adm import Adm
            Adm().sendEvent(info['id'], info['event'], info['data'])
            rsp = 'ACK'
            self.wfile.write(bytes(rsp + "\n", "utf-8"))



from subprocess import call
from threading import Thread
from ...kernel.adapter.Adapter import Adapter

class SubProcessAdapter(Adapter, Thread):

    app = None
    file = None    
    args = None

    def __init__(self, app, file, args):
        self.app = app
        self.file = file
        self.args = args
        super().__init__()

    def run(self):
        call([self.app, self.file, self.args])

    def setUp(self):
        pass
        
    def response(self, response):
        pass
    
    def request(self):
        pass

    def finalize(self):
        pass


from threading import Thread
#from ...kernel.system.Adm import Adm
from ...kernel.util.Queue import Queue
from ...kernel.adapter.Adapter import Adapter

#import web
import json

class RESTAdapter(Adapter, Thread):

    app = None
    adm = None
    conf = None
    
    def __init__(self):
        self.adm = None #Adm()
        super().__init__()

    def setUp(self):
        urls = ('/(.*)', 'RESTAdapter')
        self.app = web.application(urls, globals())
        
    def response(self, response):
        pass
    
    def request(self):
        pass

    def finalize(self):
        pass

    def run(self):
        self.app.run()

    def GET(self, name):
        if not name:
            name = 'World'
        return 'Hello, ' + name + '!'

    def POST(self, rqst):
        data = json.loads(web.data().decode('utf-8'))                        
        queue = Queue(10)        
        dto = {'queue': queue,'url': rqst, 'data': data}
        self.adm.sendEvent('front_controller', 'request', dto)
        response = queue.get()
        queue.task_done()
        web.header('Access-Control-Allow-Origin',      '*')
        web.header('Access-Control-Allow-Credentials', 'true')        
        return str(response).replace("'", "\"")






from distutils.core import setup
import setuptools

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
  name = 'pbesa',
  long_description=long_description,
  long_description_content_type='text/markdown',
  version = '3.0.7',
  license='MIT',
  description = 'An artificial intelligence platform for the implementation of multi-agent systems based on python 3 and the BESA model',
  author = 'Enrique Gonzales Guerreo, Cesar Julio Bustacara, Fabian Jose Roldan',
  author_email = 'egonzal@javeriana.edu.co, cbustaca@javeriana.edu.co, fjroldan@akenfactory.com',
  url = 'https://github.com/akenfactory/pbesa.git',
  download_url = 'https://github.com/akenfactory/pbesa/archive/3.0.7.tar.gz',
  keywords = ['agent', 'multi-agent', 'system', 'artificial', 'intelligence'],
  packages=setuptools.find_packages(),
  classifiers=[  # Optional
    # How mature is this project? Common values are
    #   3 - Alpha
    #   4 - Beta
    #   5 - Production/Stable
    'Development Status :: 5 - Production/Stable',

    # Indicate who your project is intended for
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',

    # Pick your license as you wish
    'License :: OSI Approved :: MIT License',

    # Specify the Python versions you support here. In particular, ensure
    # that you indicate whether you support Python 2, Python 3 or both.
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.8',
  ],
)



import socketserver
from threading import Thread
from ...middleware.remote.AdmListenerHandler import AdmListenerHandler

class AdmListener(Thread):

    IP = None
    PORT = None

    def __init__(self, ip, port):
        self.IP = ip
        self.PORT = port
        super().__init__()

    def run(self):        
        server = socketserver.TCPServer((self.IP, self.PORT), AdmListenerHandler)
        server.serve_forever()



from .exceptions import *
from .pooltype import PoolType
from .delegate import Delegate
from .delegateaction import DelegateAction
from .poolcontroller import PoolController
from .responseaction import ResponseAction
from .notifyfreeaction import NotifyFreeAction



from .delegateaction import DelegateAction
from .linealcontroller import LinealController
from .responseaction import ResponseAction
from .timeoutaction import TimeoutAction
from .exceptions import LinealException



from abc import abstractmethod
from .exceptions import LinealException
from ...kernel.agent.Action import Action

# --------------------------------------------------------
# Define Action
# --------------------------------------------------------
class DelegateAction(Action):
    """ An action is a response to the occurrence of an event """

    def execute(self, data):
        """ 
        Response.
        @param data Event data 
        """
        self.agent.setGateway(data['gateway'])
        self.agent.reset()
        self.delegate(data['dto'])
        
    def activeTimeout(self, time):
        self.adm.sendEvent(self.agent.id, 'timeout', {'time': time, 'command': 'start'})

    def toAssign(self, data):
        if len(self.agent.getFreeList()) > 0:
            ag = self.agent.getFreeList().pop(0)
            self.adm.sendEvent(ag, 'task', data)
        else:
            raise LinealException('[Warn, toAssign]: The number of data packets exceeds the number of agents')

    @abstractmethod
    def delegate(self, data):
        """
        Catch the exception.
        @param exception Response exception
        """
        pass




from abc import ABC, abstractmethod

class Adapter(ABC):
    
    data = None
    config = None
    
    def __init__(self):
        #Adm().addAdapter(self)
        super().__init__()

    @abstractmethod
    def setUp(self):
        pass

    @abstractmethod
    def response(self):
        pass
    
    @abstractmethod
    def request(self):
        pass

    @abstractmethod
    def finalize(self):
        pass
